/*******************************************************************

Part of the Fritzing project - http://fritzing.org
Copyright (c) 2007-2009 Fachhochschule Potsdam - http://fh-potsdam.de

Fritzing is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Fritzing is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.

********************************************************************

$Revision: 2966 $:
$Author: cohen@irascible.com $:
$Date: 2009-05-19 12:04:42 +0200 (Tue, 19 May 2009) $

********************************************************************/


#ifndef SVGFILESPLITTER_H_
#define SVGFILESPLITTER_H_

#include <QString>
#include <QByteArray>
#include <QDomElement>
#include <QObject>
#include <QMatrix>

struct PathUserData {
	QString string;
        QMatrix transform;
	qreal sNewWidth;
	qreal sNewHeight;
	qreal vbWidth; 
	qreal vbHeight;
	qreal x;
	qreal y;
};

class SvgFileSplitter : public QObject {
	Q_OBJECT

public:
	SvgFileSplitter();
	bool split(const QString & filename, const QString & elementID);
	bool splitString(QString & contents, const QString & elementID);
	const QByteArray & byteArray();
	const QDomDocument & domDocument();
	bool normalize(qreal dpi, const QString & elementID, bool blackOnly);
	QString shift(qreal x, qreal y, const QString & elementID);
	QString elementString(const QString & elementID);
    virtual bool parsePath(const QString & data, const char * slot, PathUserData &, QObject * slotTarget);

public:
	static bool getSvgSizeAttributes(const QString & path, QString & width, QString & height, QString & viewBox);


protected:
	void normalizeChild(QDomElement & childElement, 
						qreal sNewWidth, qreal sNewHeight,
						qreal vbWidth, qreal vbHeight, bool blackOnly);
	bool normalizeAttribute(QDomElement & element, const char * attributeName, qreal num, qreal denom);
	virtual void shiftChild(QDomElement & element, qreal x, qreal y);
	bool shiftAttribute(QDomElement & element, const char * attributeName, qreal d);
	void setStrokeOrFill(QDomElement & element, bool blackOnly);
	void fixStyleAttribute(QDomElement & element);
	void fixStyleAttribute(QDomElement & element, QString & style, const QString & attributeName);

protected slots:
	void normalizeCommandSlot(QChar command, bool relative, QList<double> & args, void * userData);
	void shiftCommandSlot(QChar command, bool relative, QList<double> & args, void * userData);
        virtual void rotateCommandSlot(QChar, bool, QList<double> &, void *){}

protected:
	QByteArray m_byteArray;
	QDomDocument m_domDocument;

};

#endif
